import { ElMessage, ElMessageBox } from 'element-plus';
import axios, { InternalAxiosRequestConfig, AxiosResponse } from 'axios';
import { useUserStore } from '@/stores/modules/user';
import { ST_CONFIG, LOGIN_URL } from '@/config';

// 创建 axios 实例
const axiosInstance = axios.create({
  baseURL: ST_CONFIG.APP_API_URL,
  withCredentials: false // 跨域请求时是否需要使用凭证
});

// 请求拦截器
axiosInstance.interceptors.request.use(
  (config: InternalAxiosRequestConfig) => {
    const userStore = useUserStore();
    // 添加token
    if (userStore.token) {
      (config as any).headers['token'] = userStore.token;
    }
    return config;
  },
  (error: any) => {
    return Promise.reject(error);
  }
);

// 响应拦截器
axiosInstance.interceptors.response.use(
  (response: AxiosResponse) => {
    // 处理文件流
    if (
      response.headers['content-type']?.includes('application/vnd.ms-excel') ||
      response.headers['content-type']?.includes('application/octet-stream')
    ) {
      const blob = new Blob([response.data], {
        type: response.headers['content-type']
      });
      const blobURL = window.URL.createObjectURL(blob);
      const tempLink = document.createElement('a');
      tempLink.style.display = 'none';
      tempLink.href = blobURL;
      const filename =
        JSON.parse(response.config.data).fileName ||
        (response.headers['content-disposition']
          ? decodeURI(response.headers['content-disposition'].split(';')[1].split('=')[1])
          : '未命名');
      tempLink.setAttribute('download', filename);
      if (typeof tempLink.download === 'undefined') tempLink.setAttribute('target', '_blank');
      document.body.appendChild(tempLink);
      tempLink.click();
      document.body.removeChild(tempLink);
      window.URL.revokeObjectURL(blobURL);
      return Promise.resolve(blobURL);
    }
    if (response.status === 200) {
      if (response.data.code === '00000') {
        return response.data;
      } else {
        // token过期
        if (response.data.code === 'A0301' || response.data.code === 'A0230') {
          loginHandler();
        } else {
          const reader = new FileReader();
          if (response.request.responseType && response.request.responseType == 'blob') {
            reader.onload = (e: any) => {
              const errData = JSON.parse(e.target.result);
              ElMessage.error(errData?.msg || '请求错误');
            };
            reader.readAsText(response.data);
          } else {
            ElMessage.error(response?.data?.msg || '请求错误');
          }
          return Promise.reject(response);
        }
      }
    }
  },
  (error: any) => {
    const response = error.response;
    // Network Error为 axios 在网络有问题的时候返回的字符串
    if (error && error.m === 'Network Error') {
      const errorMsg = '网络不给力, 请检查网络连接';
      return Promise.reject(new Error(errorMsg));
    }
    // 如果是被cancel掉的 是没有response的，手工生事项会cancel掉请求
    if (axios.isCancel(error)) {
      // eslint-disable-next-line prefer-promise-reject-errors
      return Promise.reject('Request canceled');
    }
    ElMessage.error(response?.data?.msg);
    throw error;
  }
);

// 导出 axios 实例
export default axiosInstance;

async function loginHandler() {
  await ElMessageBox.confirm('登录身份已失效，请重新登录!', '提示', {
    confirmButtonText: '确定',
    cancelButtonText: '取消',
    showClose: false,
    closeOnClickModal: false,
    closeOnPressEscape: false,
    showCancelButton: false
  });
  const userStore = useUserStore();
  localStorage.clear();
  userStore.setToken('');
  window.location.replace(LOGIN_URL);
}

export const midRequest = axios.create({
  baseURL: ST_CONFIG.REQUEST_MID
});

// 请求拦截器
midRequest.interceptors.request.use(config => {
  const userStore = useUserStore();
  // 添加token
  if (userStore.token) {
    config.headers['Token'] = userStore.token;
  }
  return config;
});

// 响应拦截器
midRequest.interceptors.response.use(res => {
  const code = res.data.code;
  const message = res.data.msg;
  if (res.request.responseType === 'blob' || res.request.responseType === 'arraybuffer') {
    return res.data;
  }
  if (code !== 0) {
    ElMessage.error({
      message
    });
    if (code === 9999) {
      loginHandler();
    }
    return Promise.reject(res.data);
  }
  return res.data;
});
